@@ -5,7 +5,8 @@
 
 .msg <- function(..., appendLF=TRUE, indent=0, exdent=2)
 {
-    txt <- sprintf(...)
+    contents <- list(...)
+    txt <- if (length(contents) != 1L) do.call(sprintf, contents) else contents
     message(paste(strwrap(txt, indent=indent, exdent=exdent), collapse="\n"),
         appendLF=appendLF)
 }
@@ -33,28 +34,55 @@
     message(paste(txt, collapse="\n"), appendLF=appendLF)
 }
 
+handleCondition <-
+    function(
+        ..., condition, help_text = character(0L),
+        messages = character(0L), nframe = 2L
+    )
+{
+    msg <- list(paste0(...))
+    if (!tolower(condition) %in% c("warning", "error", "note"))
+        stop("<Internal> Designate input with 'warning', 'error', or 'note'.")
+    cl <- sys.call(sys.parent(n = nframe))[[1L]]
+    ml <- structure(msg, .Names = tail(as.character(cl), 1L))
+    .BiocCheck$add(
+        ml, condition = condition, help_text = help_text, messages = messages
+    )
+    .BiocCheck$log
+}
+
 handleCheck <- function(..., appendLF=TRUE)
 {
-    msg <- paste0(...)
-    .msg("* %s", msg, appendLF=appendLF)
+    msg <- sprintf("* %s", paste0(...))
+    .BiocCheck$setCheck(msg)
+    .msg(msg, appendLF=appendLF)
 }
 
 handleError <- function(...)
 {
-    msg <- .error$add(...)
-    .msg("* ERROR: %s", msg, indent=4, exdent=6)
+    handleCondition(..., condition = "error")
+}
+
+handleErrorFiles <- function(..., help_text = "Found in files:") {
+    handleCondition(..., help_text = help_text, condition = "error")
 }
 
 handleWarning <- function(...)
 {
-    msg <- .warning$add(...)
-    .msg("* WARNING: %s", msg, indent=4, exdent=6)
+    handleCondition(..., condition = "warning")
+}
+
+handleWarningFiles <- function(..., help_text = "Found in files:") {
+    handleCondition(..., help_text = help_text, condition = "warning")
 }
 
 handleNote <- function(...)
 {
-    msg <- .note$add(...)
-    .msg("* NOTE: %s", msg, indent=4, exdent=6)
+    handleCondition(..., condition = "note")
+}
+
+handleNoteFiles <- function(..., help_text = "Found in files:") {
+    handleCondition(..., help_text = help_text, condition = "note")
 }
 
 handleMessage <- function(..., indent=4, exdent=6)
@@ -68,35 +96,46 @@ handleVerbatim <- function(msg, indent=4, exdent=6, width=getOption("width"))
     .verbatim("%s", msg, indent=indent, exdent=exdent, width=width)
 }
 
-installAndLoad <- function(pkg)
-{
-    r_libs_user_old <- Sys.getenv("R_LIBS_USER")
-    on.exit(do.call("Sys.setenv", list(R_LIBS_USER=r_libs_user_old)))
-    r_libs_user <- paste(.libPaths(), collapse=.Platform$path.sep)
-    Sys.setenv(R_LIBS_USER=r_libs_user)
-
-    dir.create(install_dir <- tempfile())
-    dir.create(libdir <- file.path(install_dir, "lib"))
-    file.create(stderr <- file.path(install_dir, "install.stderr"))
-    cmd <- file.path(Sys.getenv("R_HOME"), "bin", "R")
-    args <- sprintf("--vanilla CMD INSTALL --no-test-load --library=%s %s",
-                    libdir, shQuote(pkg))
+.run_r_command <- function(cmd, args, stderr) {
     res <- system2(cmd, args, stdout=NULL, stderr=stderr)
-    if (res != 0)
-    {
-        cat("  cmd: ", cmd,
+    if (res) {
+        message(
+            "  cmd: ", cmd,
             "\n  args: ", args,
             "\n  stderr:",
-            "\n  ", paste(readLines(stderr), collapse="\n  "),
-            "\n", sep="")
-        handleError(pkg, " must be installable.")
+            "\n  ", paste(readLines(stderr), collapse="\n  "), "\n"
+        )
     }
-    pkgname <- strsplit(basename(pkg), "_")[[1]][1]
-    args <- list(package=pkgname, lib.loc=libdir)
-    if (paste0("package:",pkgname) %in% search())
-        suppressWarnings(unloadNamespace(pkgname))
+    res
+}
 
-    suppressPackageStartupMessages(do.call(library, args))
+installAndLoad <- function(pkgpath, install_dir = tempfile())
+{
+    if (!dir.exists(install_dir))
+        dir.create(install_dir)
+    dir.create(libdir <- file.path(install_dir, "lib"))
+    file.create(stderr <- file.path(install_dir, "install.stderr"))
+
+    r_libs_user_old <- Sys.getenv("R_LIBS_USER")
+    on.exit(do.call("Sys.setenv", list(R_LIBS_USER=r_libs_user_old)))
+    r_libs_user <- paste(c(libdir, .libPaths()), collapse=.Platform$path.sep)
+    Sys.setenv(R_LIBS_USER=r_libs_user)
+
+    rcmd <- file.path(Sys.getenv("R_HOME"), "bin", "R")
+    args <- sprintf("--vanilla CMD INSTALL --no-test-load --library=%s %s",
+                    libdir, shQuote(pkgpath))
+    res <- .run_r_command(cmd = rcmd, args = args, stderr = stderr)
+    if (res) {
+        handleError(pkgpath, " must be installable.")
+    }
+    pkgname <- .getPackageName(pkgpath)
+    args <- sprintf(
+        "--vanilla -e 'library(%s)'", pkgname
+    )
+    res <- .run_r_command(cmd = rcmd, args = args, stderr = stderr)
+    if (res) {
+        handleError(pkgpath, " must be loadable.")
+    }
     install_dir
 }
 
@@ -167,57 +206,41 @@ getAllDeprecatedPkgs <- function()
     pkgs
 }
 
-parseFile <- function(infile, pkgdir)
-{
-    ## FIXME - use purl to parse RMD and RRST
-    ## regardless of VignetteBuilder value
+parseFile <- function(infile, pkgdir) {
     dir.create(parse_dir <- tempfile())
-    if (grepl("\\.Rnw$|\\.Rmd|\\.Rrst|\\.Rhtml$|\\.Rtex", infile, TRUE))
-    {
+    if (grepl("\\.Rnw$|\\.Rmd|\\.Rrst|\\.Rhtml$|\\.Rtex", infile, TRUE)) {
         outfile <- NULL
         desc <- file.path(pkgdir, "DESCRIPTION")
         dcf <- read.dcf(desc)
         if ("VignetteBuilder" %in% colnames(dcf) &&
-            dcf[,"VignetteBuilder"] == "knitr")
-        {
-            if (!requireNamespace("knitr")) {
-                stop("'knitr' package required to check knitr-based vignettes")
+            dcf[,"VignetteBuilder"] == "knitr") {
+            ## parse field in case more than one
+            vigBuilder <- unlist(
+                strsplit(dcf[, "VignetteBuilder"], ", "), use.names = FALSE
+            )
+            if ("knitr" %in% vigBuilder) {
+                if (!requireNamespace("knitr"))
+                    stop("'knitr' required to check 'Rmd' vignettes")
             }
-            outfile <- file.path(parse_dir, "parseFile.tmp")
-            # copy file to work around https://github.com/yihui/knitr/issues/970
-            # which is actually fixed but not in CRAN yet (3/16/15)
-            tmpin <- file.path(parse_dir, basename(infile))
-            file.copy(infile, tmpin)
-            suppressWarnings(suppressMessages(capture.output({
-                knitr::purl(input=tmpin, output=outfile, documentation=0L)
-            })))
-            file.remove(tmpin)
-        } else {
-            full.infile <- normalizePath(infile)
-            oof <- file.path(parse_dir, basename(infile))
-            oof <- vapply(strsplit(oof, "\\."),
-                function(x) paste(x[seq_len(length(x)-1)], collapse="."),
-                character(1))
-            outfile <- paste0(oof, '.R')
-            suppressWarnings(suppressMessages(capture.output({
-                    oldwd <- getwd()
-                    on.exit(setwd(oldwd))
-                    setwd(parse_dir)
-                    Stangle(full.infile)
-                    badname <- paste0(basename(infile), ".R")
-                    if (file.exists(badname))
-                        file.rename(badname, outfile)
-            })))
         }
-
-    } else if (grepl("\\.Rd$", infile, TRUE))
-    {
-        rd <- parse_Rd(infile)
+        outfile <- file.path(parse_dir, "parseFile.tmp")
+        suppressWarnings(suppressMessages(
+            capture.output({
+                knitr::purl(
+                    input=infile, output=outfile, documentation=0L
+                )
+            })
+        ))
+    } else if (grepl("\\.Rd$", infile, TRUE)) {
+        rd <- tools::parse_Rd(infile)
         outfile <- file.path(parse_dir, "parseFile.tmp")
         code <- capture.output(Rd2ex(rd))
         cat(code, file=outfile, sep="\n")
     } else if (grepl("\\.R$", infile, TRUE)) {
-        outfile <- infile
+        outfile <- file.path(parse_dir, "parseFile.tmp")
+        # remove empty lines before parsing
+        txt <- Filter(nzchar, readLines(infile, warn = FALSE))
+        writeLines(txt, outfile)
     }
     p <- parse(outfile, keep.source=TRUE)
     getParseData(p)
@@ -270,23 +293,86 @@ findSymbolInParsedCode <- function(parsedCode, pkgname, symbolName,
                 if (grepl("\\.R$", name, ignore.case=TRUE))
                     handleMessage(sprintf(
                         "Found %s%s in %s (line %s, column %s)", symbolName,
-                        parens, mungeName(name, pkgname), x[i,1], x[i,2]))
+                        parens, getDirFile(name), x[i,1], x[i,2]))
                 else
                     handleMessage(sprintf(
                         "Found %s%s in %s", symbolName, parens,
-                        mungeName(name, pkgname))) # FIXME test this
+                        getDirFile(name))) # FIXME test this
             }
         }
     }
     length(matches) # for tests
 }
 
-mungeName <- function(name, pkgname)
+getDirFile <- function(fpath) {
+    if (nzchar(fpath) && !is.na(fpath))
+        fpath <- file.path(basename(dirname(fpath)), basename(fpath))
+
+    fpath
+}
+
+.getTokenTextCode <- function(parsedf, token, text) {
+    parsedf[
+        parsedf$token %in% token & parsedf$text %in% text,
+        c("line1", "col1", "token", "text"),
+        drop = FALSE
+    ]
+}
+
+.grepTokenTextCode <- function(parsedf, token, text) {
+    parsedf[
+        parsedf$token %in% token & grepl(text, parsedf$text),
+        c("line1", "col1", "token", "text"),
+        drop = FALSE
+    ]
+}
+
+findSymbolsInParsedCode <-
+    function(
+        parsedCodeList, symbolNames, tokenTypes,
+        FUN = .getTokenTextCode, fun = TRUE, ...
+    )
 {
-    twoseps <- paste0(rep.int(.Platform$file.sep, 2), collapse="")
-    name <- gsub(twoseps, .Platform$file.sep, name, fixed=TRUE)
-    pos <- regexpr(pkgname, name)
-    substr(name, pos+1+nchar(pkgname), nchar(name))
+    matches <- structure(vector("list", length(parsedCodeList)),
+        .Names = names(parsedCodeList))
+    if (
+        !identical(length(tokenTypes), length(symbolNames)) &&
+        length(tokenTypes) == 1L
+    )
+        tokenTypes <- rep(tokenTypes, length(symbolNames))
+
+    for (filename in names(parsedCodeList)) {
+        df <- parsedCodeList[[filename]]
+        res <- Map(
+            function(x, y) {
+                FUN(parsedf = df, token = x, text = y, ...)
+            },
+            x= tokenTypes, y = symbolNames
+        )
+        res <- do.call(rbind.data.frame, res)
+        matches[[filename]] <- res
+    }
+
+    matches <- Filter(nrow, matches)
+    matches <- lapply(names(matches), function(nm) {
+        dframe <- matches[[nm]]
+        dframe[["text"]] <- paste0(dframe$text,
+            ifelse(dframe$token == "SYMBOL_FUNCTION_CALL", "()", ""))
+        dframe[["filename"]] <- nm
+        dframe
+    })
+
+    matches <- do.call(
+        function(...) rbind.data.frame(..., make.row.names = FALSE),
+        matches
+    )
+    apply(matches, 1L, function(rowdf) {
+        fmttxt <- "%s (line %s, column %s)"
+        formt <- if (fun) paste0(rowdf["text"], " in ", fmttxt) else fmttxt
+        sprintf(formt, getDirFile(rowdf["filename"]),
+            rowdf["line1"], rowdf["col1"]
+        )
+    })
 }
 
 isInfrastructurePackage <- function(pkgDir)
@@ -484,15 +570,21 @@ getVigSources <- function(dir)
         ignore.case=TRUE, full.names=TRUE)
 }
 
-getBadDeps <- function(pkgdir)
+getRSources <- function(Rdir) {
+    if (!identical(basename(Rdir), "R"))
+        Rdir <- file.path(Rdir, "R")
+    dir(Rdir, pattern = "\\.[Rr]$", full.names = TRUE)
+}
+
+getBadDeps <- function(pkgdir, lib.loc)
 {
     cmd <- file.path(Sys.getenv("R_HOME"), "bin", "R")
     oldquotes <- getOption("useFancyQuotes")
     on.exit(options(useFancyQuotes=oldquotes))
     options(useFancyQuotes=FALSE)
-    args <- sprintf("-q --vanilla --slave -f %s --args %s",
+    args <- sprintf("-q --vanilla --no-echo -f %s --args %s",
         system.file("script", "checkBadDeps.R", package="BiocCheck"),
-        dQuote(pkgdir))
+        paste(dQuote(pkgdir), dQuote(lib.loc)))
     system2(cmd, args, stdout=TRUE, stderr=FALSE,
         env="R_DEFAULT_PACKAGES=NULL")
 }
@@ -591,11 +683,11 @@ getParent <- function(view, biocViewsVocab)
 getFunctionLengths <- function(df)
 {
     df <- df[df$terminal & df$parent > -1,]
-    rownames(df) <- seq_len(nrow(df))
+    rownames(df) <- NULL
     max <- nrow(df)
     res <- list()
     funcRows <- df[df$token == "FUNCTION",]
-    lst<-lapply(split(df, rownames(df)), as.list)
+    lst <- lapply(split(df, rownames(df)), as.list)
     if (nrow(funcRows))
     {
         for (i in seq_len(nrow(funcRows)))
@@ -648,53 +740,14 @@ getFunctionLengths <- function(df)
             }
 
         }
+    } else {
+       res <- list(as.data.frame(list(
+           length = integer(0), startLine = integer(0), endLine = integer(0)
+       )))
     }
     res
 }
 
-doesFileLoadPackage <- function(df, pkgname)
-{
-    df <- cbind(df, idx=seq_len(nrow(df)))
-    res <- c()
-    regex <- paste0("^['|\"]*", pkgname, "['|\"]*$")
-    max <- nrow(df)
-    reqs <- df[df$token == "SYMBOL_FUNCTION_CALL" &
-        df$text %in% c("library","require"),]
-    if (nrow(reqs))
-    {
-        for (i in seq_len(nrow(reqs)))
-        {
-            reqRow <- reqs[i,]
-            currIdx <- reqs[i, "idx"]
-            if ((currIdx + 1) >= max) return(res)
-            i1 = df[df$idx == currIdx+1,]
-            p <- i1$parent
-            rowsWithThatParent <- df[df$parent == p,]
-            lastRowWithThatParent <-
-                rowsWithThatParent[nrow(rowsWithThatParent),]
-            rowsToCheck <- df[i1$idx:lastRowWithThatParent$idx,]
-            for (j in seq_len(nrow(rowsToCheck)))
-            {
-                curRow <- rowsToCheck[j,]
-                if (curRow$token %in% c("SYMBOL", "STR_CONST") &&
-                    grepl(regex, curRow$text))
-                {
-                    prevRow <- df[curRow$idx -1,]
-                    prevPrevRow <- df[curRow$idx -2,]
-                    if (!(prevRow$token == "EQ_SUB" &&
-                        prevRow$text == "=" &&
-                        prevPrevRow$token == "SYMBOL_SUB" &&
-                        prevPrevRow$text == "help"))
-                    {
-                        res <- append(res, reqRow$line1)
-                    }
-                }
-            }
-        }
-    res
-    }
-}
-
 doesManPageHaveRunnableExample <- function(rd)
 {
     hasExamples <- any(unlist(lapply(rd,
@@ -715,30 +768,13 @@ doesManPageHaveRunnableExample <- function(rd)
     length(parsed) && !inherits(parsed, "try-error")
 }
 
-
-checkLogicalUseFiles <- function(pkgdir) {
-    Rdir <- file.path(pkgdir, "R")
-    Rfiles <- dir(pkgdir, recursive=TRUE, pattern = "\\.[rR]$",
-                  full.names = TRUE)
-    dx <- startsWith(Rfiles, Rdir)
-    RdirFiles <- Rfiles[dx]
-    Rother <- Rfiles[!dx]
-    manFiles <- dir(pkgdir, recursive=TRUE, pattern = "\\.[Rr][Dd]$",
-        full.names = TRUE)
-    RNWFiles <- dir(pkgdir, recursive=TRUE, pattern = "\\.[Rr][Nn][wW]$",
-        full.names = TRUE)
-    RMDFiles <- dir(pkgdir, recursive=TRUE, pattern = "\\.[Rr][Mm][Dd]$",
-        full.names = TRUE)
-
-    allFiles <- c(Rother, RMDFiles, RNWFiles, manFiles)
-    fileNames1 <- character()
-    if (length(allFiles) > 0){
-        convertedFiles <- unlist(lapply(FUN=makeTempRFile, allFiles))
-        vl <- lapply(convertedFiles, findLogicalFile)
-        names(vl) <- c(Rother, RMDFiles, RNWFiles, manFiles)
-        badFiles <- Filter(length, vl)
-        fileNames1 <- names(badFiles)
-    }
-    sub(fileNames1, pattern=paste0(pkgdir,.Platform$file.sep),
-        replacement="", fixed=TRUE)
+selectSome <- function(obj, maxToShow = 5) {
+    stopifnot(is.character(obj))
+    if (!maxToShow %% 2)
+        stop("'maxToShow' should be an odd value")
+    edge <- (maxToShow - 1) / 2
+    if (length(obj) > maxToShow)
+        c(head(obj, edge), "...", tail(obj, edge))
+    else
+        obj
 }
